# PySNMP SMI module. Autogenerated from smidump -f python SNMP-USER-BASED-SM-3DES-MIB
# by libsmi2pysnmp-0.1.1 at Sun Nov  6 01:37:25 2011,
# Python version sys.version_info(major=2, minor=7, micro=2, releaselevel='final', serial=0)

# Imported just in case new ASN.1 types would be created
from pyasn1.type import constraint, namedval

# Imports

( Integer, ObjectIdentifier, OctetString, ) = mibBuilder.importSymbols("ASN1", "Integer", "ObjectIdentifier", "OctetString")
( snmpPrivProtocols, ) = mibBuilder.importSymbols("SNMP-FRAMEWORK-MIB", "snmpPrivProtocols")
( Bits, Integer32, ModuleIdentity, MibIdentifier, ObjectIdentity, TimeTicks, snmpModules, ) = mibBuilder.importSymbols("SNMPv2-SMI", "Bits", "Integer32", "ModuleIdentity", "MibIdentifier", "ObjectIdentity", "TimeTicks", "snmpModules")
( AutonomousType, ) = mibBuilder.importSymbols("SNMPv2-TC", "AutonomousType")

# Objects

usm3DESEDEPrivProtocol = ObjectIdentity((1, 3, 6, 1, 6, 3, 10, 1, 2, 3))
if mibBuilder.loadTexts: usm3DESEDEPrivProtocol.setDescription("The 3DES-EDE Symmetric Encryption Protocol.")
snmpUsmMIB = ModuleIdentity((1, 3, 6, 1, 6, 3, 15)).setRevisions(("1999-10-06 00:00",))
if mibBuilder.loadTexts: snmpUsmMIB.setOrganization("SNMPv3 Working Group")
if mibBuilder.loadTexts: snmpUsmMIB.setContactInfo("WG-email:   snmpv3@lists.tislabs.com\nSubscribe:  majordomo@lists.tislabs.com\n            In msg body:  subscribe snmpv3\n\nChair:      Russ Mundy\n            NAI Labs\npostal:     3060 Washington Rd\n            Glenwood MD 21738\n            USA\nemail:      mundy@tislabs.com\nphone:      +1-443-259-2307\n\nCo-editor:  David Reeder\n            NAI Labs\npostal:     3060 Washington Road (Route 97)\n            Glenwood, MD  21738\n            USA\nemail:      dreeder@tislabs.com\nphone:      +1-443-259-2348\n\nCo-editor:  Olafur Gudmundsson\n            NAI Labs\npostal:     3060 Washington Road (Route 97)\n            Glenwood, MD  21738\n            USA\nemail:      ogud@tislabs.com\nphone:      +1-443-259-2389")
if mibBuilder.loadTexts: snmpUsmMIB.setDescription("Extension to the SNMP User-based Security Model\nto support Triple-DES EDE in 'Outside' CBC\n(cipher-block chaining) Mode.")

# Augmentions

# Exports

# Module identity
mibBuilder.exportSymbols("SNMP-USER-BASED-SM-3DES-MIB", PYSNMP_MODULE_ID=snmpUsmMIB)

# Objects
mibBuilder.exportSymbols("SNMP-USER-BASED-SM-3DES-MIB", usm3DESEDEPrivProtocol=usm3DESEDEPrivProtocol, snmpUsmMIB=snmpUsmMIB)

